const state = {
    journals: [],
    loadingJournals: true,
    selectedJournal: null
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    journals(state) {
        return state.journals
    },
    /**
     * 
     * @param {*} state 
     */
    selectedJournal(state) {
        return state.selectedJournal
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setLoadingJournals(state, payload) {
        return state.loadingJournals
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setJournals(state, payload) {
        state.journals = [...payload]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    autoSelectJournal(state) {
        state.selectedJournal = state.journals.find(j => j.type === 'cash')
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setSelectedJournal(state, payload) {
        if (!payload) {
            state.selectedJournal = payload
            return
        }

        state.selectedJournal = state.journals.find(item => item.id === payload)
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setLoadingJournals(state, payload) {
        state.loadingJournals = !!payload
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    initJournals({ commit }, payload) {
        commit('setJournals', payload)
        commit('autoSelectJournal')
        commit('setLoadingJournals', false)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    selectJournal({ commit }, payload) {
        commit('setSelectedJournal', payload)
    },
    /**
     * 
     */
    resetJournal({ commit }, payload) {
        commit('setLoadingJournals', false)
        commit('setJournals', [])
        commit('setSelectedJournal', null)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}