const state = {
    paymentTerms: [],
    loadingPaymentTerms: true,
    selectedPaymentTerm: null,
    paymentType: 'cash',
    initialPayment: 0,
    paymentLines: []
}

const getters = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    paymentTerms(state) {
        return state.paymentTerms
    },
    /**
     * 
     * @param {*} state 
     */
    selectedPaymentTerm(state) {
        return state.selectedPaymentTerm
    },
    /**
     * 
     * @param {*} state 
     */
    paymentType(state) {
        return state.paymentType
    },
    /**
     * 
     * @param {*} state 
     */
    initialPayment(state) {
        return state.initialPayment
    },
    /**
     * 
     * @param {*} state 
     */
    paymentLines(state) {
        return state.paymentLines
    },
    /**
     * 
     * @param {*} state 
     */
    loadingPaymentTerms(state) {
        return state.loadingPaymentTerms
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setPaymentTerms(state, payload) {
        state.paymentTerms = [...payload]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setLoadingPaymentTerms(state, payload) {
        state.loadingPaymentTerms = !!payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    autoSelectPaymentTerm(state, payload) {
        if (!payload || payload === 'cash') {
            state.selectedPaymentTerm = state.paymentTerms.find(t => t.lines.length === 1 && t.lines[0].days === 0) || state.paymentTerms.find(t => t.lines.length === 1 && t.lines[0].days >= 0)
        } else {
            state.selectedPaymentTerm = state.paymentTerms.find(t => t.lines[0].days > 0)
        }
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setSelectedPaymentTerm(state, payload) {
        if (!payload) {
            state.selectedPaymentTerm = payload
            return
        }

        state.selectedPaymentTerm = state.paymentTerms.find(item => item.id === payload)
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setPaymentType(state, payload) {
        state.paymentType = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setInitialPayment(state, payload) {
        state.initialPayment = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setPaymentLines(state, payload) {
        state.paymentLines = []
    
        if (!payload) {
            return
        }
    
        let percentPaid = state.initialPayment / payload.total
        let distributedPercentage = -(percentPaid / state.selectedPaymentTerm.lines.length)
        let totals = []
        let residual = payload.total
        let dueDate = null
    
        for (let line of state.selectedPaymentTerm.lines) {
            dueDate = moment(payload.date).add(line.days + line.days2, 'days').format('YYYY-MM-DD')
    
            if(percentPaid && percentPaid < 1) {
                totals.push([payload.date, percentPaid])
                percentPaid = 0
    
                if (dueDate === payload.date) {
                    distributedPercentage = ((totals[0][1] - line.valueAmount) / (state.selectedPaymentTerm.lines.length - 1))
                    continue
                }
            }
    
            if (line.value !== 'balance') {
                totals.push([dueDate, line.valueAmount + distributedPercentage])
                continue
            }
    
            totals.push([dueDate, line.valueAmount])
    
            for (let line of totals) {
                let currentPrice = (payload.total * line[1]).toFixed(2)
    
                if (currentPrice < 0) {
                    continue
                }
    
                residual = residual - currentPrice
    
                state.paymentLines.push({
                    date: line[0],
                    total: currentPrice !== parseFloat(0).toFixed(2) ? currentPrice : residual.toFixed(2)
                })
            }
    
            totals = []
        }    
    }
}

const actions = {
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    initPaymentTerms({ commit }, payload) {
        commit('setPaymentTerms', payload)
        commit('autoSelectPaymentTerm')
        commit('setLoadingPaymentTerms', false)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    selectPaymentTerm({ commit }, payload) {
        if (!payload) {
            return
        }

        commit('setSelectedPaymentTerm', payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    changePaymentType({ commit }, payload) {
        commit('setPaymentType', payload),
        commit('autoSelectPaymentTerm', payload)
        commit('setInitialPayment', 0)
        commit('setPaymentLines')
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    changeInitialPayment({ commit }, payload) {
        commit('setInitialPayment', payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    computePaymentLines({ commit, getters }) {
        commit('setPaymentLines', {
            date: getters.date,
            total: getters.cartTotal
        })
    },
    /**
     * 
     * @param {*} param0 
     */
    resetPayment({ commit }) {
        commit('setLoadingPaymentTerms', true)
        commit('setPaymentTerms', [])
        commit('setSelectedPaymentTerm', null)
        commit('setPaymentType', 'cash')
        commit('setInitialPayment', 0)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}